#ifndef UNITDEF_H
#define UNITDEF_H

#include <string>
#include <utility>
#include <vector>
#include <set>
#include "unitelement.h"
#include <sbml/Model.h>

class Formula;
class Variable;

class UnitDef
{
private:
  //UnitDef(const UnitDef& src); //Undefined
  //UnitDef& operator=(const UnitDef& src); //Undefined

  std::vector<UnitElement> m_components;
  std::string m_module; //The name of the high-level module we're in.
  std::vector<std::string> m_name;
  bool m_nameautogenerated;

public:
  UnitDef(std::string name, std::string modname);
  UnitDef(std::vector<std::string> name, std::string modname);
  ~UnitDef() {};

  void SetName(std::vector<std::string> name);
  void MultiplyUnitDef(UnitDef* unitdef);
  void DivideUnitDef(UnitDef* unitdef);
  void RaiseTo(double pow);
  void MultiplyBy(double mult);
  void Reduce();
  void AddUnitElement(const UnitElement& ue);
  void ClearComponents();
  bool GetNameAutoGenerated() const;
  bool IsEmpty() const;
  bool ClearReferencesTo(Variable* deletedvar);


  bool SetFromFormula(Formula* formula);
  bool Matches(UnitDef* unitdef) const;
  bool ComponentsMatch(UnitDef* unitdef) const;
  bool IsOnlyCanonicalKind() const;

  size_t GetNumUnitElements() const;
  const UnitElement* GetUnitElement(unsigned long n) const;

  std::vector<std::string> GetName() const;
  std::string GetNameDelimitedBy(std::string cc) const;
  std::string ToStringDelimitedBy(std::string cc) const;
  void Invert();
  UnitDef* GetCanonical() const;
  UnitDef* GetCanonical(std::set<std::string> usednames) const;

  /// Returns true if is an SBML built-in unit, false otherwise
  bool IsBuiltin() const;

#ifndef NSBML
  UnitDef* GetUnitDefFromASTNode(ASTNode* astn);
  UnitDefinition* AddToSBML(Model* sbmlmod, std::string id, std::string name);
#endif

private:

};


#endif //UNITDEF_H
